package autoupdate;  # $Date: 2002/04/12 09:20:23 $

use strict;
# warnings causes problems under 5.005
#use warnings;
use POSIX;
#use RPM::Header;

BEGIN {
	use Exporter   ();
	use vars qw($VERSION $DEBUG @ISA @EXPORT @EXPORT_OK);

	# set the version for version checking
	$VERSION     = 1.20;
	@ISA         = qw(Exporter);
	@EXPORT      = qw(&SetArch &SetDistArch &GetArch &GetDistArch
			&GetAllowedArch &GetAllowedDistArch &TestArch &IsElement
			&SetLocalRPMs &AddLocalRPMs &CheckLocal &CompareLocal &SelectRPMs
			&SetQueryRpmHeaders &SetQueryRpmDatabase &TieRpmDatabase);
	@EXPORT_OK   = qw();
}
END {};

use vars qw($DEBUG $QueryRpmHeaders $QueryRpmDatabase);

$DEBUG = 0;
$QueryRpmHeaders = 0;
$QueryRpmDatabase = 0;

my $TieRpmDatabase = 0;
my $Arch;
my $DistArch;
my %LocalRPMs;
my %LocalRPMsArchs;
my @LocalTied;
my %RPM;

##################################
# Class for RPM header information
##################################

{ package Header;

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self  = {};
	my $file = "";
	my $path = "";
	my $name = "";
	my $version = "";
	my $release = "1";
	my $epoch = "";
	my $arch = "noarch";
	my $type = "r";
	my $hdr = "";
	return undef unless ($_[0]);
	$file = shift;
	chomp($file);
	if ($_[0] and $_[0] =~/^[flr]$/) {
		$type= $_[0];
	} elsif ( -f $file ) {
			$type= "f";
	}

	if ( $file =~ /^(.*\/)([^\/]+)$/ ) {
		$path = $1;
		$file = $2
	}
	if ($file =~ /^(.*)-([^-]+)-([^-]+)\.([^.]+)\.rpm$/) {
		$name= $1;
		$version= $2;
		$release= $3;
		$arch= $4;
	} elsif ($file =~ /^(.*)-(\d[^-]*)-(\d[^-]*)\.rpm$/) {
		$name= $1;
		$version= $2;
		$release= $3;
	} elsif ($file =~ /^(.*)-(\d[^-]*)\.(\D[^.]+)\.rpm$/) {
		$name= $1;
		$version= $2;
		$arch= $3;
	} elsif ($file =~ /^(.*)-(\d[^-]*)\.rpm$/) {
		$name= $1;
		$version= $2;
	}
	if ($version =~ /^(\d+):(.+)$/) {
		$epoch= $1;
		$version=$2;
	} elsif ($version =~ /^\(none\):(.+)$/) {
		$version=$1;
	}

	return undef unless($name);
	
	$self->{NAME} = $name;
	$self->{EPOCH} = $epoch;
	$self->{VERSION} = $version;
	$self->{RELEASE} = $release;
	$self->{ARCH} = $arch;
	$self->{FILENAME} = "$path$file";
	$self->{TYPE} = $type;
	$self->{HDR} = $hdr;
	bless ($self, $class);
	return $self;
}

sub getheader {
	my $self = shift;
	my $hdr;

	if ($self->{HDR}) {
		return $self->{HDR};
	}
	if ($self->{TYPE} eq "r") {
		return undef;
	} elsif ($self->{TYPE} eq "l") {
		return undef unless ($autoupdate::QueryRpmDatabase);
		&autoupdate::TieRpmDatabase(1);
		$hdr= $RPM{$self->name};
		unless ($hdr)
		{
       			print STDERR "Error: " . $self->rpmname . ": $RPM::err\n";
			$self->{TYPE} = "r";
			return undef;
		} else {
			push(@LocalTied,$self);
		}
	} elsif ($self->{TYPE} eq "f") {
		return undef unless ($autoupdate::QueryRpmHeaders);
		$hdr = RPM::Header->new( $self->filename );
		unless ($hdr)
		{
       		print STDERR "Error: " . $self->filename . ": $RPM::err\n";
			$self->{TYPE} = "r";
			return undef;
		}
	}
	$self->{NAME} = $hdr->{NAME};
	$self->{EPOCH} = $hdr->{EPOCH};
	$self->{VERSION} = $hdr->{VERSION};
	$self->{RELEASE} = $hdr->{RELEASE};
	$self->{ARCH} = $hdr->{ARCH};
	$self->{HDR} = $hdr;
	return $self->{HDR};
}

sub name {
	my $self = shift;
	return $self->{NAME};
}

sub version {
	my $self = shift;
	return $self->{VERSION}."-".$self->{RELEASE};
}

sub arch {
	my $self = shift;
	return $self->{ARCH};
}

sub rpmname {
	my $self = shift;
	return $self->{NAME}."-".$self->{VERSION}."-".$self->{RELEASE}.".".$self->{ARCH}.".rpm";
}

sub filename {
	my $self = shift;
	if ($_[0]) {
		$self->{FILENAME} = shift;
	}
	return $self->{FILENAME};
}

#
# Version comparison code
#

sub compare {
	return 0 unless ($_[0]);
	my $rpma = shift;
	return 1 unless ($_[0]);
	my $rpmb = shift;

	my $a = $rpma->{NAME};
	my $b = $rpmb->{NAME};
	if ($a ne $b) { return ($a cmp $b); }
	if ($rpma->version eq $rpmb->version) { return 0; }
	if ($autoupdate::QueryRpmHeaders and $rpma and $rpmb) {
		$a = $rpma->getheader();
		$b = $rpmb->getheader();
		if ($a and $b) {
			#Compare using rpm
			my $tmp= $a->cmpver($b);
			unless (defined($tmp)) {
				print STDERR "Error: $RPM::err\n";
			}
			return $tmp;
		}
	}

	$a = $rpma->{EPOCH};
	$b = $rpmb->{EPOCH};
	if ($a and $b) { return ($a <=> $b)};

	$a = $rpma->{VERSION};
	$b = $rpmb->{VERSION};

	if ($a eq $b) {
		#Versions are equal, need to compare the release
		#We only take the numeric part if present
		my $ra= $rpma->{RELEASE};
		my $rb= $rpmb->{RELEASE};
		# Save some time
		if ($ra eq $rb) { return 0 };

		if ( $ra=~/^(.*\d)[^.\d]*$/ ) {
			$a=$1;
			} else {
			$a=$ra;
		}
		if ( $rb=~/^(.*\d)[^.\d]*$/ ) {
			$b= $1;
		} else {
			$b=$rb;
		}
	}

	# Split off equal parts first
	my ($a1,$b1);
	while ($a ne "" and $b ne "") {
		($a1,$b1)= ("","");
		if ($a =~ /^(\d+)(.*)$/) {
			$a = $2;
			$a1 = $1;
		}
		if ($b =~ /^(\d+)(.*)$/) {
			$b = $2;
			$b1 = $1;
		}
		unless ($a1 eq $b1) {
			$a= $a1 . $a;
			$b= $b1 . $b;
			last;
		}
		($a1,$b1)= ("","");
		if ($a =~ /^(\D+)(.*)$/) {
			$a = $2;
			$a1 = $1;
		}
		if ($b =~ /^(\D+)(.*)$/) {
			$b = $2;
			$b1 = $1;
		}
		unless ($a1 eq $b1) {
			$a= $a1 . $a;
			$b= $b1 . $b;
			last;
		}
	}


	# Split off everything after a dot
	if ( $a=~/^([^.]+)/ ) {
		$a= $1;
	}
	if ( $b=~/^([^.]+)/ ) {
		$b= $1;
	}
	# Treat the case if first char is a dot
	if ( $a=~/^\.(.*)$/) {
		return 1;
	}
	if ( $b=~/^\.(.*)$/) {
		return -1;
	}

	# Now do the actual comparison

	($a1,$b1)= ("", "");
	# First compare the numeric part
	if ( $a=~/^(\d+)(.*)$/ ) {
		($a1, $a)= ($1, $2);
	}
	if ( $b=~/^(\d+)(.*)$/ ) {
		($b1, $b)= ($1, $2);
	}

	if (("$a1$b1" ne '') and ($a1 ne $b1)) {
		# a is newer than 0
		if ( ($a1 eq '') and ($b1 ne '') ) { return 1; };
		if ( ($a1 ne '') and ($b1 eq '') ) { return -1; };
		# 1 is newer than 01
		if ($a1 =~ /^0/ or $b1 =~ /^0/) {
			if ($a1 ne $b1) { return ($a1 cmp $b1); }
		}
		# standard case
		return ($a1 <=> $b1);
	}

	# Now compare the non-numeric part
	($a1,$b1)= ("", "");
	if ( $a=~/^(\D+)(.*)$/ ) {
		($a1, $a)= ($1, $2);
	}
	if ( $b=~/^(\D+)(.*)$/ ) {
		($b1, $b)= ($1, $2);
	}
	$a1 =~ s/[_-](pre|alpha|beta|rc|final)/$1/i;
	$b1 =~ s/[_-](pre|alpha|beta|rc|final)/$1/i;
	$a1 =~ s/final//i;
	$b1 =~ s/final//i;

	# 1.0xxx is newer than 1.0alpha
	if ( $b1 =~/^alpha/i ) { return 1 };
	if ( $a1 =~/^alpha/i ) { return -1 };
	# 1.0xxx is newer than 1.0beta
	if ( $b1 =~/^beta/i ) { return 1 };
	if ( $a1 =~/^beta/i ) { return -1 };
	# 1.0xxx is newer than 1.0pre
	if ( $b1 =~/^pre/i ) { return 1 };
	if ( $a1 =~/^pre/i ) { return -1 };
	# 1.0xxx is newer than 1.0rc
	if ( $b1 =~/^rc/i ) { return 1 };
	if ( $a1 =~/^rc/i ) { return -1 };


	# 1.0 is newer than 1.0a1
	if ( ($a1 eq '') and ("$b1$b"=~/^[ab]\d/i) ) { return 1 };
	if ( ("$a1$a"=~/^[ab]\d/i) and ($b1 eq '') ) { return -1 };
	# 1.0a is newer than 1.0
	if ( ($a1 eq '') and ($b1 ne '') ) { return -1 };
	if ( ($a1 ne '') and ($b1 eq '') ) { return 1 };

	# otherwise try regular string comparison
	return ($a1 cmp $b1);
}

} #END package Header


##############################
# Subs for arch
##############################

# Determine Arch
sub SetArch {
	if ($_[0]) {
		$Arch = shift;
	} else {
		$Arch=(uname())[4];
		if ($Arch eq "alpha") {
			if ( open(FILE,"</proc/cpuinfo") ) {
				my $item;
				foreach $item (<FILE>) {
					next unless $item =~ /cpu model.*ev6/i;
					$Arch="alphaev6";
					last;
				}
			close(FILE);
			}
		} elsif ($Arch eq "i686") {
			if ( open(FILE,"</proc/cpuinfo") ) {
				my $item;
				foreach $item (<FILE>) {
					next unless $item =~ /model name.*athlon/i;
					$Arch="athlon";
					last;
				}
			close(FILE);
			}
		}
	}
}

sub GetArch {
	unless ($Arch) { &SetArch };
	return $Arch;
}

# Determine DistArch
sub SetDistArch {
	if ($_[0]) {
		$DistArch = shift;
	} else {
		&SetArch unless ($Arch);
		if ( IsElement($Arch,qw(i486 i586 i686 athlon)) ) {
			$DistArch= "i386";
		} elsif ($Arch eq "sparc64") {
			$DistArch= "sparc";
		} elsif ($Arch eq "alphaev6") {
			$DistArch= "alpha";
		} else {
			$DistArch= $Arch;	
		}
	}
}

sub GetDistArch {
	unless ($DistArch) { &SetDistArch };
	return $DistArch;
}

# Determine allowedarch
sub GetAllowedArch {
	unless ($Arch) { &SetArch };
	# Safe default
	my @allowedarch= ($Arch, "noarch");

	if ( ($Arch eq "src") ) {
		@allowedarch=qw(src);
	} elsif ( ($Arch eq "i386") or ($Arch eq "i486") ) {
		@allowedarch=qw(i386 noarch);
	} elsif ( $Arch eq "i586" ) {
		@allowedarch=qw(i586 i386 noarch);
	} elsif ( $Arch eq "i686" ) {
		@allowedarch=qw(i686 i586 i386 noarch);
	} elsif ( $Arch eq "athlon" ) {
		@allowedarch=qw(athlon i686 i586 i386 noarch);
	} elsif ( $Arch eq "sparc64" ) {
		@allowedarch=qw(sparc64 sparc noarch);
	} elsif ( $Arch eq "alphaev6" ) {
		@allowedarch=qw(alphaev6 alpha noarch);
	}
	return @allowedarch;
}

# Determine AllowedDistArch
sub GetAllowedDistArch {
	unless ($DistArch) { &SetDistArch };
	# Safe default
	my @allowedarch= ($DistArch, "noarch");

	if ($DistArch eq "src") {
		@allowedarch=qw(src);
	} elsif ( IsElement($DistArch,qw(i386 i486 i586 i686 athlon)) ) {
		@allowedarch=qw(athlon i686 i586 i386 noarch);
	} elsif ( IsElement($DistArch,qw(sparc sparc64)) ) {
		@allowedarch=qw(sparc64 sparc noarch);
	} elsif ( IsElement($DistArch,qw(alpha alphaev6)) ) {
		@allowedarch=qw(alphaev6 alpha noarch);
	}
	return @allowedarch;
}

# Test if arch is known
sub TestArch {
	&SetArch unless ($Arch);
	if ( IsElement($Arch,qw(src i386 i486 i586 i686 athlon sparc sparc64 ppc alpha alphaev6 ia64)) ) {
		return 0;
	}
	print STDERR "Warning: Unknown architecture: $Arch.\n" if $_[0];
	print STDERR "Guessing valid architectures: $Arch noarch.\n" if $_[0];
	return 1;
}

###############################
# Checks if item is in an array
###############################

sub IsElement
{
	my $item;
	my $element= shift;

	foreach $item (@_) {
		return 1 if ($item eq $element);
	}
	return 0;
}

######################
# Sets the hash of local RPMs
######################

sub SetLocalRPMs
{
	%LocalRPMs= ();
	%LocalRPMsArchs= ();
	SetQueryRpmDatabase(0);
	AddLocalRPMs( @_ ) if (@_);
}

######################
# Check if a local rpm exists
######################

sub AddLocalRPMs
{
	my ($item, $tmp, $name, $res, $loc);
	my $type = "";
	if ($_[0] =~ /^[flr]$/) {
		$type= shift;
	}
	foreach $item ( @_ ) {
		$tmp = Header->new($item,$type);
		$name = $tmp->name;
		next unless $tmp;
		if ( CheckLocal($name) ) {
			$loc= $LocalRPMs{$name};
			$res= $tmp->compare($loc);
			if ($res==1) {
				$LocalRPMs{$name}= $tmp;
				$LocalRPMsArchs{$name}= [ $tmp->arch ];
			} elsif ($res ==0 and ($tmp->version eq $loc->version)) {
				push(@{$LocalRPMsArchs{$name}}, $tmp->arch); 
			}
		} else {
			$LocalRPMs{$name}=$tmp;
			$LocalRPMsArchs{$name}= [ $tmp->arch ];
		}
	}
}

######################
# Check if a local rpm exists
######################

sub CheckLocal
{
	my $name= shift;
	my $hdr= $LocalRPMs{$name};
	if ( $hdr ) {
		return $hdr->version;
	} elsif ($QueryRpmDatabase) {
		TieRpmDatabase(1);
		$hdr= $RPM{$name};
		if ($hdr) {
			my $tmp= Header->new($hdr->{NAME} . "-" . $hdr->{VERSION} . "-" . $hdr->{RELEASE} . "." . $hdr->{ARCH} . ".rpm","l");
			$tmp->{EPOCH}= $hdr->{EPOCH};
			$tmp->{HDR}= $hdr;
			$LocalRPMs{$name}= $tmp;
			push(@LocalTied,$tmp);
			return $tmp->version;
		}
	}
	return 0;
}

######################
# Compare an rpm against the local version
######################

sub CompareLocal
{
	my $name= $_[0]->name;
	my $hdr= $LocalRPMs{$name};
	unless ( $hdr or !$QueryRpmDatabase) {
		TieRpmDatabase(1);
		$hdr= $RPM{$name};
                if ($hdr) {
                        my $tmp= Header->new($hdr->{NAME} . "-" . $hdr->{VERSION} . "-" . $hdr->{RELEASE} . "." . $hdr->{ARCH} . ".rpm","l");
                        $tmp->{EPOCH}= $hdr->{EPOCH};
                        $tmp->{HDR}= $hdr;
                        $LocalRPMs{$name}= $tmp;
                        push(@LocalTied,$tmp);
                        $hdr=$tmp;
                }
	}
	return 1 unless ($hdr);
	return $_[0]->compare($hdr);
}

######################
# Should we try to query rpm headers
######################

sub SetQueryRpmHeaders {
	if (defined($_[0]) and ($_[0] ne $QueryRpmHeaders) ) {
		$QueryRpmHeaders= $_[0];
		if ($QueryRpmHeaders) {
			eval "use RPM::Header";
			if ($@) { 
				warn('Failed to load RPM::Header');
				$QueryRpmHeaders= 0;
			}
		}
	}
	return $QueryRpmHeaders;
}

######################
# Should we try to query the rpm db
######################

sub SetQueryRpmDatabase {
	if (defined($_[0]) and ($_[0] ne $QueryRpmDatabase)) {
		$QueryRpmDatabase= $_[0];
		if ($QueryRpmDatabase) {
			eval "use RPM::Header";
			if ($@) { 
				warn('Failed to load RPM::Header');
				$QueryRpmDatabase= 0;
			}
		}
		TieRpmDatabase($QueryRpmDatabase);
	}
	return $QueryRpmDatabase;
}

######################
# Tie/Untie the rpm database
######################

sub TieRpmDatabase {
	return 0 if ($_[0] eq $TieRpmDatabase);
	$TieRpmDatabase=0;
	unless ($_[0]) {
		print "Closing rpm db.\n" if $DEBUG;
		foreach my $item (@LocalTied) {
			$item->{HDR}= "";
			if ($LocalRPMs{$item->name} and $LocalRPMs{$item->name} eq $item) {
				delete $LocalRPMs{$item->name};
			}
		}
		@LocalTied= ();
		untie(%RPM);
		return 0;
	}
	print "Opening rpm db.\n" if $DEBUG;
	unless ( tie(%RPM, "RPM::Database") ) {
		print STDERR "Fatal: Could not tie rpm database.\n";
		die ($RPM::err);
	}
	$TieRpmDatabase=1;
	return 0;
}

######################
# Select rpms from a given array of rpms
# Returns array of selected rpms 
######################

sub SelectRPMs
{
	my $ref= shift;
	my @rpms =();
	my %Version=();
	my %Test=();
	my ($item, $rpm, $name, $arch, $rver, $dver, $ldver, $tmp, $res, $loc);
	my $BestMatch=1;
	my $CheckLocal=0;

	if (defined $_[0]) {
			$BestMatch= shift;
			if (defined $_[0]) {
				$CheckLocal= shift;
			}
	}

	# Get a list of allowed architectures
	my @AllowedArch;
	if ($BestMatch ==1 ) {
		@AllowedArch= GetAllowedArch();
	} else {
		@AllowedArch= GetAllowedDistArch();
	}
	print "AUPM: Selecting rpms (best=$BestMatch,local=$CheckLocal,arch=`@AllowedArch`):\n" if $DEBUG;
	# BestMatch=1  => Selects best arch if more than one is available
	# BestMatch=0  => Selects all arch if more than one is available
	# CheckLocal=0 => Do not check against local version
	# CheckLocal=1 => Exclude rpms with older or no local version
	# CheckLocal=2 => Exclude rpms with older local version

	# Loop through all rpms
	# Remember latest version in hash
	foreach $rpm ( @{$ref} ) {
		print "AUPM: " . $rpm->rpmname . " " if $DEBUG;
		if ( defined $Test{$rpm->rpmname} ) {
			print "(already present)\n" if $DEBUG;
			next;
		} else {
			$Test{$rpm->rpmname}= 1;
		}		
		unless ( IsElement($rpm->{ARCH}, @AllowedArch) ) {
			print "(wrong arch)\n" if $DEBUG;
			next;
		}
		$name=$rpm->name;		
		if ( $CheckLocal ) { # Check against local version
			unless ( CheckLocal($name) or $CheckLocal == 2 ) {
				print "(no local version)\n" if $DEBUG;
				next;
			}
			$loc= $LocalRPMs{$name};
			$res= $rpm->compare($loc);
			if ( $res == -1 or ($res == 0 and ($BestMatch or ($rpm->version ne $loc->version) )) ) {
				print "(have local version)\n" if $DEBUG;
				next;
			} elsif ($res ==0 and IsElement($rpm->arch, @{$LocalRPMsArchs{$name}})) {
				print "(have local version)\n" if $DEBUG;
				next;
			}
		}
		if ($Version{$name}) {
			$tmp = $rpm->compare($Version{$name}->[0]);
		} else {
			$tmp = 1;
		}

		if ( $tmp == 1 ) { # Newer then what we currently have
			print "(new)\n" if $DEBUG;
			$Version{$name}= [ $rpm ];
		} elsif ( $tmp == 0) { # Same then what we currently have
			print "(same)\n" if $DEBUG;
			push(@{$Version{$name}}, $rpm);
		} else {
			print "(old)\n" if $DEBUG;
		}
	}

	if ($BestMatch) { #Select best match
		foreach $name (keys %Version) {
			my @tmp= @{$Version{$name}};
			next if (@tmp == 1);

			#Select best version
			# Try to match the nonnumeric part of the release
			# This usually stands for the distribution
			$tmp= $LocalRPMs{$name};
			if ( $tmp and $tmp->{RELEASE} =~ /^[^-]+-.*\d([^.\d]*)$/ ) {
				$ldver= $1;
			} else {
				$ldver= "";
			}
			$rpm= $tmp[0];
			($rver, $dver)= ($rpm->{RELEASE}, "");
			if ( $rver =~/^([^-]+-.*\d)([^.\d]*)$/ ) {
				($rver, $dver)= ($1, $2);
			}
			foreach $item ( @tmp ) {
				if ( $item->{RELEASE} eq "$rver$ldver" ) {
					$rpm= $item;
					last;
				} elsif ( $item->{RELEASE} eq $rver ) {
					$rpm= $item;
					last;
				}
			}

			@{$Version{$name}}= ();
			foreach $item ( @tmp ) {
				if ( $item->{RELEASE} eq $rpm->{RELEASE} ) {
					push(@{$Version{$name}}, $item);
				}
			}

			#Select best arch
			$rpm= undef;
			foreach $arch ( @AllowedArch ) { # Array ordered by preference!
				last if ($rpm);
				foreach $item ( @{$Version{$name}} ) {
					next unless ( $item->{ARCH} eq $arch );
					$rpm= $item;
					last;
				}
			}
			$Version{$name}= [ $rpm ];
		}
	}

	foreach $name (sort keys %Version) {
		push @rpms, @{$Version{$name}};
	}
	return @rpms;
}


1;

__END__

=head1 NAME

autoupdate.pm

=head1 SYNOPSIS

 $rpm = Header->new("/path/foo-1.0-1.i386.rpm");
 $rpm->name
 $rpm->version
 $rpm->arch
 $rpm->rpmname
 $rpm->filename
 $rpm1->compare($rpm2)

 &SetArch &SetDistArch &TestArch
 &GetArch &GetDistArch &GetAllowedArch &GetAllowedDistArch
 &SetLocalRPMs &AddLocalRPMs &CheckLocal &CompareLocal &SelectRPMs
 &SetQueryRpmHeaders &SetQueryRpmDatabase &TieRpmDatabase
 &IsElement

=head1 DESCRIPTION

This module implements an (stripped down) rpm header object plus some functions to
manipulate them.

=head2 Header objects

A header object can be crated with

 $rpm = Header->new("/path/foo-1.0-1.i386.rpm");

from a file name. An optional argument "f", "r", or "l" can
be used to specify the type.

 "f(ile)" the rpm corresponds to an existing file and can be
         queried using the rpm bindings.
 "l(ocal)" the rpm corresponds to a locally installed rpm
 "r(emote)" the rpm corresponds to a remote file which cannot
         be queried

The string shouold be of the form "[/path/]name-[epoch:]version-release.arch.rpm".

Methods implemented are

 $rpm->name     returns the rpm name (e.g.: foo)
 $rpm->version  returns the rpm version (e.g.: 1.0-1)
 $rpm->arch     returns the rpm architecutre (e.g.: i386)
 $rpm->rpmname  returns the full rpm name (e.g.: foo-1.0-1.i386.rpm)
 $rpm->filename returns the full file name (e.g.: /path/foo-1.0-1.i386.rpm)

 $rpm1->version($rpm2) compares the versions of $rpm1 and $rpm2.

Each object contains the following date:

 $rpm->{NAME}     name tag of the rpm
 $rpm->{EPOCH}    epoch tag of the rpm
 $rpm->{VERSION}  version tag of the rpm
 $rpm->{RELEASE}  release tag of the rpm
 $rpm->{ARCH}     arch tag of the rpm
 $rpm->{FILENAME} the full file name
 $rpm->{TYPE}     the type (f|l|c)
 $rpm->{HDR}      pointer to the header object returned by the rpm bindings

=head2 Functions

First of all there the following functions related to architectures defined:

 &SetArch([$string])       Sets the architecture to the given value or (if none is
                           given) tries to determine it from the current machine.
 &SetDistArch([$string])   Same for distribution architecture
 &GetArch                  Returns the architecture
 &GetDistArch              Returns the distribution architecture
 &GetAllowedArch           Returns an array of allowed architectures
 &GetAllowedDistArch       Returns an array of allowed architectures
 &TestArch                 Checks if the arhitecture is known

 &SetLocalRPMs(@rpms)      Takes an array of file names and builds a hash of local                                      rpms
 &AddLocalRPMs(@rpms)      Adds file names to the hash of local rpms
 &CheckLocal($name)        Checks if an rpm is in the hash of local rpms and returns
                           its version
 &CompareLocal($rpm)       Compares an rpm against the local version and returns the
                           result

 &SelectRPMs(\@rpms, 0|1, 0|1|2) Selects rpms from the given array by comparing them
                           to the local hash. Only the rpm with the highest version
                           are selected. If the second argument is 1, only the rpm is
                           selected for each version which matches best. If the third
                           argument is 1 only rpms which are newer than the local one
                           are selected, if it is 2 only rpms which are newer than the
                           local one or for which no local version exists are selected.

 &SetQueryRpmHeaders(0|1)  Should rpms (of type file) be queried using the rpm                                          bindings.
 &SetQueryRpmDatabase(0|1) Should the local rpm data base be queried using the rpm
                           bindings.
 &TieRpmDatabase(0|1)      Tie, untie the local rpm data base.
 
 &IsElement($item,@array)  Tests if $item is element of @array.

=head1 SEE ALSO

perl(1), rpm(8), RPM(3)

=head1 AUTHOR

Gerald Teschl

=head1 LICENSE

GPL

=cut
